#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon May  7 13:28:37 2018
Contain the hydraulic physical model and the cost function that calculates the p.o.f of the quantity of interest.

@author: Jerome Stenger
"""
import numpy as np
from pyDOE import *


def my_model(x):
	x.append(30)
	x.append(50)
	x.append(54.5)
	return (x[0]/((300*x[1]*((x[3]-x[2])**0.5))/((5000)**0.5)))**0.6



def Cost_Function(x,**kwargs): #the vector x as a specific format [[],[],...,[]] so the length of the vector is the dimension of the problem
	a=0;
	lower = kwargs.get('lower', None)
	if not(np.any(lower)):
		lower = len(x)*[0] #default lower bound
	upper = kwargs.get('upper', None)
	if not(np.any(upper)):
		upper = len(x)*[1] #default upper bound
	mean = kwargs.get('mean',None)
	if not(np.any(mean)):
		mean = [(upper[i]+lower[i])/2. for i in range(len(x))] #defaut mean
	threshold=kwargs.get('threshold',None)
	if not(np.any(threshold)):
		threshold = 0 #defaut threshold
	if len(lower)!=len(x):
		print('lower bounds must be same length as x')
	p=np.ones(len(x))
	n=0
	size=np.prod([len(x[i]) for i in range(len(x))]) # Size of the n-D grid
	C=np.ones(size) # Weight of every point of the grid in n-D
	R=np.transpose(np.matrix(mean))*np.matrix(np.ones(size)) # coordinate of every point of the n-D grid
	ff=(ff2n([len(x[i]) for i in range(len(x))].count(2))+1)/2 # creation of the grid by making every combinaison possible
	
	for i in range(len(x)):
		if (len(x[i])==1):
			if (x[i][0]>mean[i]):
				p[i]=((mean[i]-lower[i])/float((x[i][0]-lower[i]))) #if the variable has one point simulated the best probability is calculated regarding the inf
			elif (x[i][0]<mean[i]):
				p[i]((mean[i]-upper[i])/float((x[i][0]-upper[i])))
			
			C=C*p[i]
			R[i,:]=x[i][0]*np.ones(len(C))
			
		elif (len(x[i])==2):
			if (x[i][1]!=x[i][0]):
				p[i]=((mean[i]-x[i][0])/float((x[i][1]-x[i][0]))) # Weight of the Dirac mass located on [mean, upper]
				
			R[i,:]=x[i][1]+(x[i][0]-x[i][1])*(ff[:,n])
			C=C*(p[i]+(1-2*p[i])*(ff[:,n]))
			n+=1
		else:
			print('solution cannot be supported on more than 2 diracs')

	for k in range(len(C)):
		a=a+C[k]*(my_model([R[:,k].tolist()[i][0] for i in range(len(R[:,k]))])>=threshold) #add the weight of each point inside the set of failure.
		
	return 1-a
